'use strict';

/*
 * The syntax parsing algorithm in this file is copied heavily from Lea Verou's
 * Prism project, which bears this license:
 *
 * MIT LICENSE
 *
 * Copyright (c) 2012-2013 Lea Verou
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
*/

let grammars = require('./grammars');
exports.grammars = grammars;

exports.highlight = function highlight(source, grammarName, opts = {}) {
  let grammar;

  if (typeof grammarName === 'string') {
    grammar = grammars[grammarName];
  }

  if (typeof grammarName === 'object' && grammarName.constructor === Object) {
    grammar = grammarName;
  }

  if (!grammar) {
    throw new Error(`Grammar "${grammarName}" not found.`);
  }

  return stringify(encode(tokenize(source, grammar)), opts.lineClass);
};

function tokenize(source, grammar) {
  let strings = [source];
  let inlines = grammar.inlines;

  if (inlines) {
    // jshint -W089
    for (let inlineToken in inlines) {
      grammar[inlineToken] = inlines[inlineToken];
    }

    delete grammar.inlines;
  }

  for (let token in grammar) {
    let patterns = grammar[token];

    if (!Array.isArray(patterns)) {
      patterns = [patterns];
    }

    for (let i = 0; i < patterns.length; i++) {
      let pattern = patterns[i];
      let { block, classes, inside } = pattern;
      let lookbehind = !!pattern.lookbehind;
      let lookbehindLength = 0;
      let tag;

      if (pattern.tag) {
        tag = pattern.tag;
      } else {
        tag = block ? 'div' : 'span';
      }

      pattern = pattern.pattern || pattern;

      for (let i = 0; i < strings.length; i++) {
        let string = strings[i];

        if (typeof string !== 'string') {
          continue;
        }

        pattern.lastIndex = 0;

        let match = pattern.exec(string);

        if (!match) {
          continue;
        }

        if (lookbehind) {
          lookbehindLength = match[1].length;
        }

        let from = match.index - 1 + lookbehindLength;
        match = match[0].slice(lookbehindLength);
        let len = match.length;
        let to = from + len;
        let before = string.slice(0, from + 1);
        let after = string.slice(to + 1);
        let args = [i, 1];

        if (before) {
          args.push(before);
        }

        let wrapped = {
          block  : block,
          classes: classes,
          tag    : tag,
          text   : inside ? tokenize(match, inside) : match,
          token  : token
        };

        args.push(wrapped);

        if (after) {
          args.push(after);
        }

        Array.prototype.splice.apply(strings, args);
      }
    }
  }

  return strings;
}

function stringify(source, lineClass = 'md-line') {
  if (source === '\n') {
    return '<br>';
  }

  if (typeof source === 'string') {
    return source.replace(/\n/g, '');
  }

  if (Array.isArray(source)) {
    return source.reduce(function addItem(result, item) {
      return result + stringify(item, lineClass);
    }, '');
  }

  let { tag } = source;
  let classList = source.classes || source.token.split(' ');
  let className = classList.map((token) => `md-${token}`).join(' ');

  if (source.block) {
    className += ` ${lineClass}`;
  }

  return [
    `<${tag} class="${className}">`,
      stringify(source.text, lineClass),
    `</${tag}>`
  ].join('');
}

function encode(src) {
  if (Array.isArray(src)) {
    return src.map(encode);
  } else if (typeof src === 'object') {
    src.text = encode(src.text);
    return src;
  } else {
    return escapeHTML(src);
  }
}

function escapeHTML(str) {
  return str
    .replace(/&/g, '&amp;')
    .replace(/>/g, '&gt;')
    .replace(/</g, '&lt;')
    .replace(/"/g, '&quot;')
    .replace(/'/g, '&#x27;')
    .replace(/\//g, '&#x2F;');
}
