/* jshint mocha: true, -W101 */

'use strict';

require('6to5ify/node_modules/6to5-core/register');

var miniprism = require('../src/miniprism');
var expect = require('chai').expect;

describe('miniprism', function() {
  it('accepts a custom lineClass', function() {
    var actual = miniprism.highlight('test', 'markdown', { lineClass: 'Line' });
    expect(actual).to.eql('<div class="md-p Line">test</div>');
  });

  it('accepts custom grammars', function() {
    var actual = miniprism.highlight('foo !bar!', { test: /!.*!/g });
    expect(actual).to.eql('foo <span class="md-test">!bar!</span>');
  });

  [1, 2, 3, 4, 5, 6].forEach(function testHeading(level) {
    it('parses h' + level + ' headings', function() {
      var source = strTimes('#', level) + ' Hello';
      var actual = miniprism.highlight(source, 'markdown');
      expect(actual).to.eql([
        '<div class="md-h' + level + ' md-has-folding md-line">',
          '<span class="md-hash md-folding">' + strTimes('#', level) + ' </span>',
          'Hello',
        '</div>'
      ].join(''));
    });
  });

  it('does not parse "h7" headings', function() {
    var source = '####### Hello world';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to
      .eql('<div class="md-p md-line">####### Hello world</div>');
  });

  it('parses unordered list items', function() {
    var source = '- Item';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-ul md-has-folding md-line">',
        '<span class="md-ul-marker md-folding">- </span>Item',
      '</div>'
    ].join(''));
  });

  it('parses ordered list items', function() {
    var source = '1. Item';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-ol md-line">',
        '<span class="md-ol-marker">1. </span>Item',
      '</div>'
    ].join(''));
  });

  it('parses blockquotes', function() {
    var source = '> Item';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-blockquote md-line">',
        '<span class="md-blockquote-marker">&gt; </span>Item',
      '</div>'
    ].join(''));
  });

  it('parses blank lines', function() {
    var source = 'Foo\n\n\nbar\n\nbaz\nqux';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">Foo</div>',
      '<div class="md-blank md-line"><br></div>',
      '<div class="md-blank md-line"><br></div>',
      '<div class="md-p md-line">bar</div>',
      '<div class="md-blank md-line"><br></div>',
      '<div class="md-p md-line">baz</div>',
      '<div class="md-p md-line">qux</div>',
    ].join(''));
  });

  it('parses blank lines in the last position', function() {
    var source = 'Foo\n';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">Foo</div>',
      '<div class="md-blank md-line"><br></div>'
    ].join(''));
  });

  it('allows for headers to follow paragraphs', function() {
    var source = 'Foo\n# Header';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">Foo</div>',
      '<div class="md-h1 md-has-folding md-line">',
        '<span class="md-hash md-folding"># </span>Header',
      '</div>'
    ].join(''));
  });

  it('can highlight indented code blocks', function() {
    var source = '    Foo\n      Bar\nbaz';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-code md-line">    Foo</div>',
      '<div class="md-code md-line">      Bar</div>',
      '<div class="md-p md-line">baz</div>'
    ].join(''));
  });

  it('can highlight fenced code blocks', function() {
    var source = '```ruby\nclass Foo\n\nend\n```\nbar';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-gfm md-line">',
        '<div class="md-gfm-limit md-line">',
          '<span class="md-gfm-fence">```</span>',
          '<span class="md-gfm-info">ruby</span>',
        '</div>',
        '<div class="md-gfm-line md-line">class Foo</div>',
        '<div class="md-gfm-line md-line"><br></div>',
        '<div class="md-gfm-line md-line">end</div>',
        '<div class="md-gfm-limit md-line">',
          '<span class="md-gfm-fence">```</span>',
        '</div>',
      '</div>',
      '<div class="md-p md-line">bar</div>'
    ].join(''));
  });

  it('parses h1 setex headers', function() {
    var source = 'Hello\n=';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-setex-h1 md-line">',
        '<div class="md-setex-header md-line">Hello</div>',
        '<div class="md-setex-marker md-line">=</div>',
      '</div>'
    ].join(''));
  });

  it('parses h2 setex headers', function() {
    var source = 'Hello\n-';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-setex-h2 md-line">',
        '<div class="md-setex-header md-line">Hello</div>',
        '<div class="md-setex-marker md-line">-</div>',
      '</div>'
    ].join(''));
  });

  it('does not parse list items as setex headers', function() {
    var source = '- Hello\n-';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-ul md-has-folding md-line">',
        '<span class="md-ul-marker md-folding">- </span>Hello',
      '</div>',
      '<div class="md-p md-line">-</div>'
    ].join(''));
  });

  ['*', '-', '_'].forEach(function(ch) {
    it('parses horzontal rules with "' + ch + '"', function() {
      var source = ch + ch + ch;
      var actual = miniprism.highlight(source, 'markdown');
      expect(actual).to.eql(
        '<div class="md-hr md-line">' + ch + ch + ch + '</div>'
      );
    });
  });

  ['*', '_'].forEach(function(ch) {
    it('parses ems with "' + ch + '"', function() {
      var source = 'foo ' + ch + 'bar' + ch + ' baz';
      var actual = miniprism.highlight(source, 'markdown');
      expect(actual).to.eql([
        '<div class="md-p md-line">',
          'foo ',
          '<span class="md-em md-has-folding">',
            '<span class="md-em-marker md-folding">' + ch + '</span>',
            'bar',
            '<span class="md-em-marker md-folding">' + ch + '</span>',
          '</span>',
          ' baz',
        '</div>'
      ].join(''));
    });
  });

  ['*', '_'].forEach(function(ch) {
    it('parses strongs with "' + ch + '"', function() {
      var source = 'foo ' + ch + ch + 'bar' + ch + ch + ' baz';
      var actual = miniprism.highlight(source, 'markdown');
      expect(actual).to.eql([
        '<div class="md-p md-line">',
          'foo ',
          '<span class="md-strong md-has-folding">',
            '<span class="md-strong-marker md-folding">' + ch + ch + '</span>',
            'bar',
            '<span class="md-strong-marker md-folding">' + ch + ch + '</span>',
          '</span>',
          ' baz',
        '</div>'
      ].join(''));
    });
  });

  it('parses links', function() {
    var source = 'Go to: [Example page](https://www.example.com).';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">',
        'Go to: ',
        '<span class="md-link md-has-folding">',
          '<span class="md-bracket-start md-folding">[</span>',
          '<span class="md-ref">Example page</span>',
          '<span class="md-bracket-end md-folding">]</span>',
          '<span class="md-paren-start md-folding">(</span>',
          '<span class="md-href md-folding">https:&#x2F;&#x2F;www.example.com</span>',
          '<span class="md-paren-end md-folding">)</span>',
        '</span>',
      '.</div>'
    ].join(''));
  });

  it('parses link references', function() {
    var source = 'This is a *[link][reference]*.';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">This is a ',
        '<span class="md-em md-has-folding">',
          '<span class="md-em-marker md-folding">*</span>',
          '<span class="md-link-ref md-has-folding">',
            '<span class="md-ref-text">',
              '<span class="md-bracket-start md-folding">[</span>',
              '<span class="md-ref">link</span>',
              '<span class="md-bracket-end md-folding">]</span>',
            '</span>',
            '<span class="md-ref-value md-folding">',
              '<span class="md-bracket-start">[</span>',
              '<span class="md-ref-href">reference</span>',
              '<span class="md-bracket-end">]</span>',
            '</span>',
          '</span>',
          '<span class="md-em-marker md-folding">*</span>',
        '</span>.',
      '</div>'
    ].join(''));
  });

  it('parses link definitions', function() {
    var source = '[name]: https://www.example.com';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-link-def md-line">',
        '<span class="md-link-def-name">',
          '<span class="md-bracket-start">[</span>',
          'name',
          '<span class="md-bracket-end">]</span>',
          '<span class="md-colon">:</span>',
        '</span>',
        '<span class="md-link-def-value"> https:&#x2F;&#x2F;www.example.com</span>',
      '</div>'
    ].join(''));
  });

  it('parses inline code', function() {
    var source = 'foo `bar` baz';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-p md-line">foo ',
        '<span class="md-inline-code md-has-folding">',
          '<span class="md-inline-code-marker md-folding">`</span>',
          'bar',
          '<span class="md-inline-code-marker md-folding">`</span>',
        '</span>',
      ' baz</div>'
    ].join(''));
  });

  it('does not parse ``` as inline code', function() {
    var source = 'foo ``` baz';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql('<div class="md-p md-line">foo ``` baz</div>');
  });

  it('escapes contents', function() {
    var source = '> foo <bar&"\'\/';
    var actual = miniprism.highlight(source, 'markdown');
    expect(actual).to.eql([
      '<div class="md-blockquote md-line">',
        '<span class="md-blockquote-marker">&gt; </span>',
        'foo &lt;bar&amp;&quot;&#x27;&#x2F;',
      '</div>'
    ].join(''));
  });
});

function strTimes(str, x) {
  var res = '';

  for (var i = 0; i < x; i++) {
    res += str;
  }

  return res;
}
